@@ -1,4 +1,4 @@
-/* $Id: dccm.c,v 1.18 2004/07/09 05:23:55 twogood Exp $ */
+/* $Id: dccm.c,v 1.21 2005/03/27 03:10:03 juno106 Exp $ */
 #include <synce_socket.h>
 #include <synce_log.h>
 #include <stdbool.h>
@@ -69,6 +69,7 @@ typedef struct _Client
 static Client * current_client = NULL;
 static bool running = true;
 static bool is_daemon = true;
+static bool exit_on_disconnect = false;
 static bool allow_root = false;
 static char* password = NULL;
 
@@ -135,7 +136,7 @@ static void write_help(char *name)
 			stderr, 
 			"Syntax:\n"
 			"\n"
-			"\t%s [-d LEVEL] [-f] [-h] [-p PASSWORD]\n"
+			"\t%s [-d LEVEL] [-f [-s]] [-h] [-p PASSWORD]\n"
 			"\n"
 			"\t-d LEVEL     Set debug log level\n"
 			"\t                 0 - No logging\n"
@@ -143,6 +144,7 @@ static void write_help(char *name)
 			"\t                 2 - Errors and warnings\n"
 			"\t                 3 - Everything\n"
 			"\t-f           Do not run as daemon\n"
+			"\t-s           Exit on device disconnect (only with -f)\n"
 			"\t-h           Show this help message\n"
 			"\t-p PASSWORD  Use this password when device connects\n"
       "\t-r           Allow dccm to run as root - USE AT YOUR OWN RISK!\n",
@@ -157,7 +159,7 @@ static bool handle_parameters(int argc, char** argv)
 	int c;
 	int log_level = SYNCE_LOG_LEVEL_ERROR;
 
-	while ((c = getopt(argc, argv, "d:fhp:r")) != -1)
+	while ((c = getopt(argc, argv, "d:fhp:rs")) != -1)
 	{
 		switch (c)
 		{
@@ -177,6 +179,10 @@ static bool handle_parameters(int argc, char** argv)
 					free(password);
 				password = strdup(optarg);
 				break;
+				
+			case 's':
+				exit_on_disconnect = true;
+				break;
 
       case 'r':
         allow_root = true;
@@ -189,6 +195,15 @@ static bool handle_parameters(int argc, char** argv)
 		}
 	}
 
+	/* -s sanity check */
+	if (is_daemon && exit_on_disconnect)
+	{
+		fprintf (stderr,
+		         "The \'-s\' option only makes sense when running in foreground.\n\n");
+		write_help(argv[0]);
+		return false;
+	}
+
 	synce_log_set_level(log_level);
 
 	return true;
@@ -290,6 +305,8 @@ static bool client_write_file(Client* client)
 {
 	bool success = false;
 	char* filename = NULL;
+	char *mode_str;
+	mode_t file_mode;
 	FILE* file = NULL;
 	char ip_str[16];
 
@@ -359,7 +376,20 @@ static bool client_write_file(Client* client)
 			
 exit:
 	if (file)
+	{
 		fclose(file);
+		
+		if ((mode_str = getenv ("SYNCE_CONNECTION_FILE_MODE")))
+		{
+			if (strlen (mode_str))
+			{
+				if (sscanf (mode_str, "%o", &file_mode) == 1)
+				{
+					chmod (filename, file_mode);
+				}
+			}
+		}
+	}
 	if (filename)
 		free(filename);
 	return success;
@@ -635,6 +665,8 @@ bool write_pid_file(const char* filename)
 	bool success = false;
 	struct stat dummy;
 	char pid_str[16];
+	char *mode_str;
+	mode_t file_mode;
 	FILE* file = NULL;
 
 	if (0 == stat(filename, &dummy))
@@ -676,6 +708,17 @@ bool write_pid_file(const char* filename)
 	fputs(pid_str, file);
 	fclose(file);
 	file = NULL;
+	
+	if ((mode_str = getenv ("SYNCE_PID_FILE_MODE")))
+	{
+		if (strlen (mode_str))
+		{
+			if (sscanf (mode_str, "%o", &file_mode) == 1)
+			{
+				chmod (filename, file_mode);
+			}
+		}
+	}
 
 	success = true;
 
@@ -719,13 +762,6 @@ int main(int argc, char** argv)
 	snprintf(pid_file, sizeof(pid_file), "%s/" DCCM_PID_FILE, path);
 	free(path);
 
-	if (!write_pid_file(pid_file))
-	{
-		goto exit;
-	}
-
-	wrote_pid_file = true;
-
 	if (password)
 	{
 		int i;
@@ -758,6 +794,13 @@ int main(int argc, char** argv)
 	else
 		synce_trace("Running in foreground");
 
+	if (!write_pid_file(pid_file))
+	{
+		goto exit;
+	}
+
+	wrote_pid_file = true;
+
 	/* signal handling */
 	signal(SIGPIPE, SIG_IGN);
 	signal(SIGHUP, handle_sighup);
@@ -828,6 +871,10 @@ int main(int argc, char** argv)
       if (client.authenticated)
         run_scripts("disconnect");
     }
+    
+		if (exit_on_disconnect)
+			running = false;
+
 	}
 
 	run_scripts("stop");
@@ -846,4 +893,3 @@ exit:
 
 	return result;
 }
-
